import json
from copy import deepcopy
from types import ModuleType

from requests import session

from . import resources
from . import schemas
from .utils import capitalize_camel_case
from .version import VERSION

RESOURCE_CLASSES = {}
SCHEMAS = {}

for name, module in resources.__dict__.items():
    capitalized_name = capitalize_camel_case(name)
    is_module = isinstance(module, ModuleType)
    is_in_module = capitalized_name in getattr(module, "__dict__", {})
    if is_module and is_in_module:
        RESOURCE_CLASSES[name] = module.__dict__[capitalized_name]

class Culqi:
    def __init__(self, public_key, private_key):
        self.public_key = public_key
        self.private_key = private_key
        self.session = session()

        self._set_client_headers()

        for name, klass in RESOURCE_CLASSES.items():
            setattr(self, name, klass(self))

    @staticmethod
    def _get_version():
        return ".".join(VERSION)

    @staticmethod
    def _update_request(data, options):
        """Update The resource data and header options."""
        data = json.dumps(data)

        if "headers" not in options:
            options["headers"] = {}

        options["headers"].update(
            {"Content-type": "application/json", "Accept": "application/json"}
        )

        return data, options

    def _set_client_headers(self):
        self.session.headers.update(
            {
                "User-Agent": "Culqi-API-Python/{0}".format(self._get_version()),
                "Authorization": "Bearer {0}".format(self.private_key),
                "Content-type": "application/json",
                "Accept": "application/json",
            }
        )

    def request(self, method, url, **options):
        """Dispatch a request to the CULQUI HTTP API."""
        response = getattr(self.session, method)(url, **options)

        data = response.json()

        if "data" in data:
            data["items"] = deepcopy(data["data"])
            del data["data"]

        print(data)

        return {"status": response.status_code, "data": data}

    def get(self, url, params, **options):
        return self.request("get", url, params=params, **options)

    def post(self, url, data, **options):
        data, options = self._update_request(data, options)
        return self.request("post", url, data=data, **options)

    def patch(self, url, data, **options):
        data, options = self._update_request(data, options)
        return self.request("patch", url, data=data, **options)

    def delete(self, url, data, **options):
        data, options = self._update_request(data, options)
        return self.request("delete", url, data=data, **options)

    def put(self, url, data, **options):
        data, options = self._update_request(data, options)
        return self.request("put", url, data=data, **options)



SCHEMA = {
    "type": "object",
    "properties": {
        "amount":           {"type" : "number"},
        "charge_id":        {"type" : "string"},
        "reason":           {"type" : "string"},
    },
    "required": ["amount", "charge_id", "reason"]
}


from ..utils.errors import ErrorMessage, NotAllowedError
from ..utils.urls import URL
from .base import Resource

__all__ = ["Token"]


class Token(Resource):
    endpoint = URL.TOKEN

    def create(self, data, **options):
        url = "https://secure.culqi.com/v2/tokens"
        
        headers = {"Authorization": "Bearer {0}".format(self.client.public_key)}
        if "headers" in options:
            options["headers"].update(headers)
        else:
            options["headers"] = headers

        return self._post(url, data, **options)

    def delete(self, id_, data=None, **options):
        raise NotAllowedError(ErrorMessage.NOT_ALLOWED)



SCHEMA = {
    "type": "object",
    "properties": {
        "card_id":      {"type" : "string"},
        "plan_id":      {"type" : "string"},
        "metadata":     {"type" : "object"}
    },
    "required": ["card_id", "plan_id"]
}


from requests.compat import urljoin
from jsonschema import validate

from ..utils.urls import URL

__all__ = ["Resource"]


class Resource:
    endpoint = None

    def __init__(self, client=None):
        self.client = client

    def _get(self, url, data, **kwargs):
        return self.client.get(url, data, **kwargs)

    def _patch(self, url, data, **kwargs):
        return self.client.patch(url, data, **kwargs)

    def _post(self, url, data, **kwargs):
        return self.client.post(url, data, **kwargs)

    def _put(self, url, data, **kwargs):
        return self.client.put(url, data, **kwargs)

    def _delete(self, url, data, **kwargs):
        return self.client.delete(url, data, **kwargs)

    def _get_url(self, *args):
        return urljoin(
            URL.BASE,
            "/".join([URL.VERSION, self.endpoint] + [str(arg) for arg in args]),
        )

    def create(self, data, **options):
        if (hasattr(self, 'schema')):
            validate(instance=data, schema=self.schema)
        url = self._get_url()
        return self._post(url, data, **options)

    def list(self, data=None, **options):
        url = self._get_url()
        return self._get(url, data, **options)

    def read(self, id_, data=None, **options):
        url = self._get_url(id_)
        return self._get(url, data, **options)

    def update(self, id_, data=None, **options):
        url = self._get_url(id_)
        return self._patch(url, data, **options)

    def delete(self, id_, data=None, **options):
        url = self._get_url(id_)
        return self._delete(url, data, **options)



from ..utils.errors import ErrorMessage, NotAllowedError
from ..utils.urls import URL
from .base import Resource
from ..schemas import charge

__all__ = ["Charge"]


class Charge(Resource):
    schema = charge.SCHEMA
    endpoint = URL.CHARGE

    def delete(self, id_, data=None, **options):
        raise NotAllowedError(ErrorMessage.NOT_ALLOWED)

    def capture(self, id_, data=None, **options):
        url = self._get_url(id_, "capture")
        return self._post(url, data, **options)



SCHEMA = {
    "type": "object",
    "properties": {
        "amount":               {"type" : "number"},
        "capture":              {"type" : "boolean"},
        "installments":         {"type" : "number"},
        "currency_code":        {"type" : "string"},
        "email":                {"type" : "string"},
        "description":          {"type" : "string"},
        "source_id":            {"type" : "string"},
        "metadata":             {"type" : "object"},
        "antifraud_details":     {
            "type": "object",
            "properties": {
                "address":      {"type" : "string"},
                "address_city": {"type" : "string"},
                "country_code": {"type" : "string"},
                "first_name":   {"type" : "string"},
                "last_name":    {"type" : "string"},
                "phone_number": {"type" : "number"},
            },
        },
    },
    "required": ["amount", "currency_code", "email", "source_id"]
}


from ..utils.urls import URL
from .base import Resource
from ..schemas import subscription

__all__ = ["Subscription"]


class Subscription(Resource):
    schema = subscription.SCHEMA
    endpoint = URL.SUBSCRIPTION



from ..utils.errors import ErrorMessage, NotAllowedError
from ..utils.urls import URL
from .base import Resource

__all__ = ["Transfer"]


class Transfer(Resource):
    endpoint = URL.TRANSFER

    def create(self, data, **options):
        raise NotAllowedError(ErrorMessage.NOT_ALLOWED)

    def update(self, id_, data=None, **options):
        raise NotAllowedError(ErrorMessage.NOT_ALLOWED)

    def delete(self, id_, data=None, **options):
        raise NotAllowedError(ErrorMessage.NOT_ALLOWED)



from ..utils.urls import URL
from .base import Resource
from ..schemas import order

__all__ = ["Order"]


class Order(Resource):
    schema = order.SCHEMA
    endpoint = URL.ORDER

    def confirm(self, id_, data=None, **options):
        url = self._get_url(id_, "confirm")
        return self._post(url, data, **options)



SCHEMA = {
    "type": "object",
    "properties": {
        "first_name":       {"type" : "string"},
        "last_name":        {"type" : "string"},
        "email":            {"type" : "string"},
        "address":          {"type" : "string"},
        "address_city":     {"type" : "string"},
        "country_code":     {"type" : "string"},
        "phone_number":     {"type" : "string"},
        "metadata":         {"type" : "object"}
    },
    "required": ["first_name", "last_name", "email", "address", "address_city", "country_code", "phone_number"]
}


class URL:
    BASE = "https://api.culqi.com"
    VERSION = "v2"

    CARD = "cards"
    CHARGE = "charges"
    CUSTOMER = "customers"
    EVENT = "events"
    ORDER = "orders"
    PLAN = "plans"
    REFUND = "refunds"
    SUBSCRIPTION = "subscriptions"
    TOKEN = "tokens"

    TRANSFER = "transfers"
    IIN = "iins"



SCHEMA = {
    "type": "object",
    "properties": {
        "name":             {"type" : "string"},
        "amount":           {"type" : "number"},
        "currency_code":    {"type" : "string"},
        "interval":         {"type" : "string"},
        "interval_count":   {"type" : "number"},
        "trial_days":       {"type" : "number"},
        "limit":            {"type" : "number"},
        "metadata":         {"type" : "object"}
    },
    "required": ["name", "amount", "currency_code", "interval", "interval_count"]
}


SCHEMA = {
    "type": "object",
    "properties": {
        "amount":                   {"type" : "number"},
        "currency_code":            {"type" : "string"},
        "description":              {"type" : "string"},
        "order_number":             {"type" : "string"},
        "client_details":     {
            "type" : "object",
            "properties": {
                "first_name":       {"type" : "string"},
                "last_name":        {"type" : "string"},
                "email":            {"type" : "string"},
                "phone_number":     {"type" : "string"},
            }
        },
        "expiration_date":          {"type" : "number"},
        "confirm":                  {"type" : "boolean"},
        "metadata":                 {"type" : "object"},
    },
    "required": ["amount", "currency_code", "description", "order_number", "expiration_date", "client_details"]
}


from ..utils.errors import ErrorMessage, NotAllowedError
from ..utils.urls import URL
from .base import Resource
from ..schemas import refund

__all__ = ["Refund"]


class Refund(Resource):
    schema = refund.SCHEMA
    endpoint = URL.REFUND

    def delete(self, id_, data=None, **options):
        raise NotAllowedError(ErrorMessage.NOT_ALLOWED)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

from setuptools import find_packages, setup

package_name = "culqi"
package_path = os.path.abspath(os.path.dirname(__file__))
repositorty_url = "https://github.com/culqi/culqi"
long_description_file_path = os.path.join(package_path, "README.md")
long_description = ""

try:
    with open(long_description_file_path) as f:
        long_description = f.read()
except IOError:
    pass


setup(
    name=package_name,
    packages=find_packages(exclude=[".*", "docs", "scripts", "tests*", "legacy",]),
    include_package_data=True,
    version=__import__("culqi").__version__,
    description="""Biblioteca de Culqi en Python""",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Willy Aguirre - Culqi Team",
    author_email="willy.aguirre@culqi.com",
    zip_safe=False,
    keywords=["Api Client", "Payment Integration", "Culqi", "Python 3", "Python 2",],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    url=repositorty_url,
    download_url="%(url)s/-/archive/%(version)s/%(package)s-%(version)s.tar.gz"
    % {
        "url": repositorty_url,
        "version": __import__("culqi").__version__,
        "package": package_name,
    },
    requires=["requests",],
    install_requires=["requests",],
)



SCHEMA = {
    "type": "object",
    "properties": {
        "customer_id":  {"type" : "string"},
        "token_id":     {"type" : "string"},
        "validate":     {"type" : "boolean"},
        "metadata":     {"type" : "object"}
    },
    "required": ["customer_id", "token_id"]
}


from .card import Card
from .charge import Charge
from .customer import Customer
from .event import Event
from .iin import Iin
from .order import Order
from .plan import Plan
from .refund import Refund
from .subscription import Subscription
from .token import Token
from .transfer import Transfer

__all__ = [
    "Card",
    "Charge",
    "Customer",
    "Event",
    "Iin",
    "Order",
    "Plan",
    "Refund",
    "Subscription",
    "Token",
    "Transfer",
]



# -*- encoding: utf-8 -*-
class ErrorCode:
    """Codigos de Denegación de Bancos.

    - EXPIRED_CARD
      Tarjeta vencida. La tarjeta está vencida o la fecha de vencimiento
      ingresada es incorrecta.

    - STOLEN_CARD
      Tarjeta robada. La tarjeta fue bloqueada y reportada al banco emisor
      como una tarjeta robada.

    - LOST_CARD
      Tarjeta perdida. La tarjeta fue bloqueada y reportada al banco emisor
      como una tarjeta perdida.

    - INSUFFICIENT_FUNDS
      Fondos insuficientes. La tarjeta no tiene fondos suficientes para
      realizar la compra.

    - CONTACT_ISSUER
      Contactar emisor. La operación fue denegada por el banco emisor de la
      tarjeta y el cliente necesita contactarse con la entidad para conocer
      el motivo.

    - INVALID_CVV
      CVV inválido. El código de seguridad (CVV2, CVC2, CID) de la tarjeta es
      inválido.

    - INCORRECT_CVV
      CVV incorrecto. El código de seguridad (CVV2, CVC2, CID) de la tarjeta
      es incorrecto.

    - TOO_MANY_ATTEMPTS_CVV
      Exceso CVV. El cliente ha intentado demasiadas veces ingresar el código
      de seguridad (CVV2, CVC2, CID) de la tarjeta.

    - ISSUER_NOT_AVAILABLE
      Emisor no disponible. El banco que emitió la tarjeta no responde. El
      cliente debe realizar el pago nuevamente.

    - ISSUER_DECLINE_OPERATION
      Operación denegada. La operación fue denegada por el banco emisor de la
      tarjeta por una razón desconocida.

    - INVALID_CARD
      Tarjeta inválida. La tarjeta utilizada tiene restricciones para este tipo
      de compras. El cliente necesita contactarse con el banco emisor para
      conocer el motivo de la denegación.

    - PROCESSING_ERROR
      Error de procesamiento. Ocurrió un error mientras procesabamos la compra.
      Contáctate con culqi.com/soporte para que te demos una solución.

    - FRAUDULENT
      Operación fraudulenta. El banco emisor de la tarjeta sospecha que se
      trata de una compra fraudulenta.

    - CULQI_CARD
      Tarjeta Culqi. Estás utilizando una tarjeta de pruebas de Culqi para
      realizar una compra real.
    """

    EXPIRED_CARD = "expired_card"
    STOLEN_CARD = "stolen_card"
    LOST_CARD = "lost_card"
    INSUFFICIENT_FUNDS = "insufficient_funds"
    CONTACT_ISSUER = "contact_issuer"
    INVALID_CVV = "invalid_cvv"
    INCORRECT_CVV = "incorrect_cvv"
    TOO_MANY_ATTEMPTS_CVV = "too_many_attempts_cvv"
    ISSUER_NOT_AVAILABLE = "issuer_not_available"
    ISSUER_DECLINE_OPERATION = "issuer_decline_operation"
    INVALID_CARD = "invalid_card"
    PROCESSING_ERROR = "processing_error"
    FRAUDULENT = "fraudulent"
    CULQI_CARD = "culqi_card"


class HTTPErrorCode:
    """Tipos de Errores.

    - INVALID_REQUEST_ERROR:
      HTTP 400 - La petición tiene una sintaxis inválida.

    - AUTHENTICATION_ERROR:
      HTTP 401 - La petición no pudo ser procesada debido a problemas con las
      llaves.

    - PARAMETER_ERROR:
      HTTP 422 - Algún parámetro de cualquier petición es inválido.

    - CARD_ERROR:
      HTTP 402 - No se pudo realizar el cargo a una tarjeta.

    - LIMIT_API_ERROR:
      HTTP 429 - Estás haciendo muchas peticiones rápidamente al API o
      superaste tu límite designado.

    - RESOURCE_ERROR:
      HTTP 404 - El recurso no puede ser encontrado, es inválido o tiene un
      estado diferente al permitido.

    - API_ERROR:
      HTTP 500 y 503 - Engloba cualquier otro tipo de error (Ejemplo: problema
      temporal con los servidores de Culqi) y debería de ocurrir muy pocas
      veces.
    """

    # HTTP 400
    INVALID_REQUEST_ERROR = "invalid_request_error"
    # HTTP 401
    AUTHENTICATION_ERROR = "authentication_error"
    # HTTP 422
    PARAMETER_ERROR = "parameter_error"
    # HTTP 402
    CARD_ERROR = "card_error"
    # HTTP 429
    LIMIT_API_ERROR = "limit_api_error"
    # HTTP 404
    RESOURCE_ERROR = "resource_error"
    # HTTP 500 503
    API_ERROR = "api_error"


class ErrorMessage:
    NOT_ALLOWED = "You can't perform this action."


class BadRequestError(Exception):
    pass


class GatewayError(Exception):
    pass


class ServerError(Exception):
    pass


class NotAllowedError(Exception):
    pass



from .errors import (
    BadRequestError,
    ErrorCode,
    ErrorMessage,
    GatewayError,
    HTTPErrorCode,
    NotAllowedError,
    ServerError,
)
from .status_codes import HTTPStatusCode
from .urls import URL


def capitalize_camel_case(string):
    return "".join([item.capitalize() for item in string.split("_")])
